#!/usr/bin/env python

import sys
import random

"""
Created by: Alex Laird
Purpose: Project #3 in Survey of Programming Languages class
Date Created: 3/30/09
Filename: sudokuSolver.py
Version: 0.1

The following is an application that will solve a Sudoku puzzle to any degree
of difficulty received from an input file.  The input file may be specified via
an argument when launching the program, or if an argument is not specified the
default filename "sudoku.txt" is used.

The file should be in the following format:

5 3 - | - 7 - | - - -
6 - - | 1 9 5 | - - -
- 9 8 | - - - | - 6 -
---------------------
8 - - | - 6 - | - - 3
4 - - | 8 - 3 | - - 1
7 - - | - 2 - | - - 6
---------------------
- 6 - | - - - | 2 8 -
- - - | 4 1 9 | - - 5
- - - | - 8 - | - 7 9

Valid numbers are integers from 1 to 9, and a blank position is specified by a
dash (-).

This application solves using no efficiency algorithms, simply through brute
force deduction.
"""

"""
If an improper operation is performed when attempting to solve the Sudoku
puzzle, or if the board given was not proper for some reason, this exception
will be created and thrown.
"""
class SudokuException(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return str(self.value)

"""
This class is the representation of the Sudoku puzzle given.  It contains the
methods for building the puzzle object, solving the puzzle object, and
creating a string representation of the puzzle object for display.
"""
class SudokuPuzzle:
    """
    This method will fill the inidivual box within the puzzle at location
    (x, y) with the value represented in the value variable (if specified).
    """
    def fillBox(self, x, y, value = None):
        # ensure that the specified value is valid
        if value != None and (value < 1 or value > 9):
            raise SudokuException("Invalid value " + str(value) + " at (" + str(x) + ", " + str(y) + ") is invalid.")
        
        # ensure that there is not already a value in the specified box
        if self.sudoku[x][y] != None:
            raise SudokuException("Board position (" + str(x) + ", " + str(y) + ") is already filled.")

        # brute force search for the proper value to be placed in the current
        # box, ensuring the value is not already taken in the current row,
        # column, or square of the puzzle
        pos = [x for x in range(1, 10)]
        
        # ensure the value is not in the row
        for scanX in range(9):
            if self.sudoku[scanX][y] != None and self.sudoku[scanX][y] in pos:
                pos.remove(self.sudoku[scanX][y])

        # ensure the value is not in the column
        for scanY in range(9):
            if self.sudoku[x][scanY] != None and self.sudoku[x][scanY] in pos:
                pos.remove(self.sudoku[x][scanY])

        # ensure the value is not in the current square
        boxX = x - x % 3
        boxY = y - y % 3
        for scanX in range(boxX, boxX + 3):
            for scanY in range(boxY, boxY + 3):
                if self.sudoku[scanX][scanY] != None and self.sudoku[scanX][scanY] in pos:
                    pos.remove(self.sudoku[scanX][scanY])

        # if a value is specified, use that
        if value == None:
            # if this is the last empty box in the row, column, or square, fill it
            if len(pos) == 1:
                self.sudoku[x][y] = pos[0]
            else:
                raise SudokuException("(" + str(x) + ", " + str(y) + ") does not need to be filled.")
        # place specified value in position, if legal
        else:
            if value in pos:
                self.sudoku[x][y] = value
            else:
                raise SudokuException("Invalid value " + str(value) + " at (" + str(x) + ", " + str(y) + ") is invalid.")

        # return the box
        return self.sudoku[x][y]

    """
    This method will solve the Sudoku puzzle, if possible.
    """
    def solve(self):
        filled = True
        
        # ensure a value value is in every box
        while filled:
            filled = False

            for y in range(9):
                for x in range(9):
                    try:
                        v = self.fillBox(x, y)
                    except SudokuException as e:
                        pass
                    else:
                        filled = True

    """
    This constructs the SudokuPuzzle object with 81 boxes set to None.
    """
    def __init__(self):
        self.sudoku = [[None for x in range(9)] for y in range(9)]

    """
    This method returns the string representation of the SudokuPuzzle object.
    """
    def __str__(self):
        string = ""
        # print each row
        for x in range(9):
            # horizontal divisions (end of square row)
            if x != 0 and x % 3 == 0:
                string += "---------------------\n"

            # print each columns item in the row
            for y in range(9):
                # vertical divisions (end of square column)
                if y != 0 and y % 3 == 0:
                    string += "| "

                # if no value is given, insert two blank spaces
                if self.sudoku[x][y] == None:
                    string += "  "
                # value is given, so place value
                else:
                    string += str(self.sudoku[x][y])
                    if y < 8:
                         string += " "

            # end of the row reached
            string += "\n"

        # return the string
        return string

"""
This method will display the initial and the completed puzzle.
"""
def display(sudoku):
    # display the original puzzle (print the str representation of the
    # SudokuPuzzle object
    print("Original Puzzle:")
    print(sudoku)

    # solve the puzzle
    sudoku.solve()
    # display the completed puzzle
    print("Completed Puzzle:")
    print(sudoku)

"""
This method executes the Sudoku solver, accepting the input, finding the
proper values for each box, and producing the solved Sudoku for the output.
"""
def main(args):
    # declare variables needed
    x = 0
    inputStream = None
    sudoku = SudokuPuzzle()

    # if command line arguments are present, use that as the input file
    if(len(args) >= 2):
        inputStream = open(args[1], "r")
    # use default input file
    else:
        inputStream = open("sudoku.txt", "r")

    # parse each line of the input file
    for line in inputStream:
        y = 0
        # skip horizontal division lines
        if line[:3] == "---":
            continue

        # break the line into components separated by spaces
        line = line.split(" ")

        # parse each value separated by spaces in the row
        for value in line:
            # skip vertical division lines
            if value == "|":
                continue
            if value == "-":
                # increment column counter            
                y += 1
                continue
            if value == "-\n":
                break

            # fill the (x, y) box with the value from the input
            sudoku.fillBox(x, y, int(value))

            # increment column counter            
            y += 1

        # increment row counter
        x += 1
        
    """
    for i in range(100):
        try:
            x = random.randrange(0, 8)
            y = random.randrange(0, 8)
            value = random.randrange(1, 9)
            print(x, y, value)
            sudoku.fillBox(x, y, value)
        except SudokuException as e:
            pass
    """

    # display the original and completed puzzle
    display(sudoku)

"""
Check for the main method and run it, passing it the system arguments
from the shell.
"""
if __name__ == "__main__":
    exitCond = main(sys.argv)
    sys.exit(exitCond)
    
